#!/usr/bin/perl -w 
#
#
#
# take parse blast data files and re-write into tab-delimited format
#   query name, frame
#   
# OUTPUT columns...
#   #1, query name
#   #2, frame, in standard format (-3 to 3)
#
#

# CHANGES
# 10/21/13, rmf
#   updated usage instructions
#   added proper command line switches and switch processing
#   
#




use strict;

my $usage 							= qq (
translation_frame_from_blast_table.pl -i infile [-o outfilebase] [-e E value] [-b bit score]

Will read in custom tab-delimted, parsed blast data and will output query name
and reading frame based on 1st hit / hsp alignment. Hits/HSPs above the E value and/or
below the bit score will be ignored. 

	-i      input file
	-o      output filename          [default: inputfile + '.tln_frame.dat']
	-e      E value cutoff          [default: 10 ]
	-b      bit score cutoff        [default: 0]
			
	-h      this help text

);


# set defaults and parse args...
my ($infile, $datfile, $ignoredfile);
my $Evalue              = 10;
my $bitscore            = 0;
my $datfile_suffix      = '.tln_frame.dat';
my $ignoredfile_suffix  = '.ignored';

# and parse args
parse_args();


my %seen;

open (my $IN, "<$infile") or die "Error opening file $infile for reading: $!\n";
open (my $DAT, ">$datfile") or die "Error opening file $datfile for writing: $!\n";
open (my $IGNORE, ">$ignoredfile") or die "Error opening file $ignoredfile for writing: $!\n";

while (my $line = <$IN>) {
	next if $line =~ m/^\s|#/;
	
	# remove newline and grab only three columns
	chomp $line;
	#my ($query_name, $strand, $frame);
	#if ($parse_flag eq 'table') {
	#    ($query_name, $strand, $frame) = (split /\t/, $line)[1,17,18];
	#}
	#else {
	my ($query_name, $hit_Evalue, $hit_bitscore, $strand, $frame) = (split /\t/, $line)[1,4,10,17,18];
	#}
	# calculate real frame and output
	if (defined ($seen{$query_name})) {
	    # if we've already seen this query, skip this line
	    next;
	}
	else {
	    if ($hit_Evalue <= $Evalue && $hit_bitscore > $bitscore) {
	        my $query_frame = ($frame + 1) * $strand;
    	    print {$DAT} $query_name, "\t", $query_frame, "\n";
    	}
    	else {
    	    print {$IGNORE} $line, "\n";
    	}
    	$seen{$query_name} = 1;
	}
	# throw away 2nd line
	#$line = <$IN>;
}

close $IGNORE;
close $DAT;
close $IN;

exit;

#
#
####

sub parse_args {
	# parse arguments
	while (scalar(@ARGV)) {
		my $arg						= shift @ARGV ;
		if 	  ($arg eq '-h') 		{ die $usage;                    } 
		elsif ($arg eq '-i') 		{ $infile 		= shift @ARGV;   }
		elsif ($arg eq '-o') 		{ $datfile	 	= shift @ARGV;   }
		elsif ($arg eq '-e') 		{ $Evalue    	= shift @ARGV;   }
		elsif ($arg eq '-b') 		{ $bitscore 	= shift @ARGV;   }
		else 						{ die "unknown argument '$arg'"; }
	}
	
	# -i input file
	if (!defined $infile) {
		print "You must supply an input filename using '-i' argument!\n";
		die $usage;
	};
	
	# -o output file basename
	# if no -o, then trim ending of -i
	if (!defined $datfile) {
	    $datfile = $infile . $datfile_suffix;
	}
	
	# and now prep ignored filename...
	$ignoredfile = $datfile . $ignoredfile_suffix;


}
